﻿using System;
using System.Data;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using NHibernate.Context;
using DataAccessLayer;
using NHibernate;

namespace WebFormsHarness
{
    public class NHibernateSessionPerConversationModule : IHttpModule
    {

        #region Constants

        const string NHIBERNATE_CONVERSATION_END_FLAG_KEY = "hibernate.conversation.end.flag";

        const string NHIBERNATE_CURRENT_SESSION_KEY = "hibernate.session";

        #endregion

        #region Public Methods

        public void Dispose()
        {
        }

        public static void SetEndConversationFlag()
        {
            HttpContext.Current.Items[NHIBERNATE_CONVERSATION_END_FLAG_KEY] = true;
        }

        public void Init(HttpApplication context)
        {
            context.PreRequestHandlerExecute += new EventHandler(Application_BeginRequest);
            context.PostRequestHandlerExecute += new EventHandler(Application_EndRequest);
        }

        #endregion

        #region Private Methods

        private void Application_BeginRequest(object sender, EventArgs e)
        {
            ISession currentSession = (ISession)HttpContext.Current.Session[NHIBERNATE_CURRENT_SESSION_KEY];
            if (currentSession == null)
            {
                currentSession = StaticSessionManager.OpenSession();
                currentSession.FlushMode = FlushMode.Never;
            }
            CurrentSessionContext.Bind(currentSession);
            currentSession.BeginTransaction();
        }

        private void Application_EndRequest(object sender, EventArgs e)
        {
            ISession session = CurrentSessionContext.Unbind(StaticSessionManager.SessionFactory);

            if (HttpContext.Current.Items[NHIBERNATE_CONVERSATION_END_FLAG_KEY] != null)
            {
                session.Flush();
                session.Transaction.Commit();
                session.Close();
                HttpContext.Current.Session[NHIBERNATE_CURRENT_SESSION_KEY] = null;
            }
            else
            {
                session.Transaction.Commit();
                HttpContext.Current.Session[NHIBERNATE_CURRENT_SESSION_KEY] = session;
            }

        }

        #endregion

    }
}